/* Socket Wrapping functions */
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include <errno.h>
#include <strings.h>
#include <string.h>
#include <stdio.h>

#include <unistd.h>
#include <syslog.h>

#include "general.h"
#include "sock.h"

#define SERV_PORT	4567

extern int h_errno;
extern OCSPD_CONFIG *ocspd_conf;

int Listen (char *hostname, int port, int queue_len) {
	int fd, reuse_addr;
	struct sockaddr_in servaddr;
	struct hostent *ht = NULL;

	if((fd = socket(AF_INET, SOCK_STREAM, 0)) == 0) {
		syslog( LOG_ERR, "ERROR::SOCKET::Initializing Socket [%s]",
			strerror(errno));
		return(0);
	};

	reuse_addr = 1;
	if(setsockopt(fd, SOL_SOCKET,  SO_REUSEADDR,  &reuse_addr, 
				sizeof(reuse_addr)) == -1 ) {
		syslog( LOG_ERR, "ERROR::SOCKET::Set Socket Options [%s]",
			strerror(errno));
		return(0);
	}
	memset( &servaddr, '\x0', sizeof( servaddr ));

	servaddr.sin_family = AF_INET;
	if( (hostname == NULL ) || (hostname[0] == '*')) {
		servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	} else {
		if((ht = gethostbyname( hostname )) == NULL ) {
			syslog( LOG_ERR, "ERROR::SOCKET::Get Hostname [%s]",
				strerror(h_errno));
			return(0);
		}
		/*
		if( ht->h_addr_list == NULL ) {
			fprintf(stderr, "%s:%d::ERROR::%s\n",
				__FILE__, __LINE__, "No address associated.");
		}
		*/

		servaddr.sin_addr = 
				(**(struct in_addr **)ht->h_addr_list);
	}

	if( ocspd_conf->verbose )
		syslog (LOG_INFO, "INFO::Local Address %s [%d]\n", 
	 		inet_ntoa(servaddr.sin_addr), port );

	if ( port > 0 ) {
		servaddr.sin_port = htons( port );
	} else {
		servaddr.sin_port = htons( SERV_PORT );
	}

	if( bind(fd, (struct sockaddr *) &servaddr, sizeof(servaddr)) == -1 ) {
		syslog(LOG_ERR, "ERROR::SOCKET::Bind [%s]",
			strerror( errno ));
		return(0);
	};

	if (listen(fd, queue_len) == -1) {
		syslog(LOG_ERR, "FATAL ERROR::Listen::%s",
			strerror( errno ));
		return(0);
	};

	return(fd);
}

int Accept (int listen_sockfd, struct sockaddr * cliaddr, socklen_t *addrlenp) {

	int n;
	struct sockaddr addr;
	socklen_t len;

again:

	if( !cliaddr || !addrlenp ) {
		len=sizeof(addr);
		n = accept(listen_sockfd, &addr, &len);
	} else {
		n = accept(listen_sockfd, cliaddr, addrlenp);
	}
		
	if ( n < 0) {
		if (INTERRUPTED_BY_SIGNAL)
			goto again;

		syslog(LOG_ERR,"ERROR::Socket::ACCEPT::%s",
			strerror( errno ));
  	}
	return(n);
}

/*
int Accept ( int sock, int *c_sock ) {

	if(( *c_sock = accept( sock, NULL, NULL )) == -1 ) {
		fprintf(stderr, "%s:%d::Accept::ERROR [%d::%s]\n", 
			__FILE__, __LINE__, errno, strerror(errno));
		return ( -1 );
	}

	return ( 0 );
}
*/

ssize_t Read (int fd, void *bufptr, size_t nbytes) {

	ssize_t n;

again:
	if ( (n = read(fd,bufptr,nbytes)) < 0) {
		if (INTERRUPTED_BY_SIGNAL)
			goto again;
		else
			syslog(LOG_ERR, "ERROR::Socket Read failed!");
	}
	return(n);
}

int Select (int maxfdp1, fd_set *readset, fd_set *writeset, 
			fd_set *exceptset, struct timeval *timeout) {

	int n;
again:
	if ( (n = select (maxfdp1, readset, writeset, 
				exceptset, timeout)) < 0) {
		if (INTERRUPTED_BY_SIGNAL)
			goto again;
		else
			syslog( LOG_ERR, "ERROR::Select failed!\n");
  	}
	return(n);
}

